@@ -1,5 +1,5 @@
 /* 
- * libCW- 2.0 - Copyright (C) 2006 -> 2011 by Ted Williams - WA0EIR
+ * libCW- 2.0 - Copyright (C) 2006 -> 2014 by Ted Williams - WA0EIR
  * Derived mostly from qrq.c - High speed morse trainer
  * by Fabian Kurz, DJ1YFK, Copyright (C) 2006, and
  * a few bits from unixcw
@@ -43,37 +43,43 @@
 #include <pulse/simple.h>
 #include <pulse/error.h>
 
+/*
+ * set these as needed by the application
+ */
 #define TESTMODE  0
-#define MESG      0       /* 0 = no message , 1 =  with message support */
+#define MESG      0               /* 0=no message, 1=with message support */
 #define DEBUG     0
 
-#define FUDGE     1.066   /* fudge factor for my lousy soundcard */
+#define FUDGE     1.000           /* no fudge factor */
+#if 0
+#define FUDGE     1.066           /* fudge factor for my lousy soundcard */
+#endif
 
 #define MAX_NAME  20
 #define MAX_LETTERS 512
-#define MAX_WORD_LENGTH  100L    /* Maximum length of a word */
-#define PI 3.1415926             /* Indiana residents, feel free to */
-                                 /* change it to 3.2 */
+#define MAX_WORD_LENGTH  100L     /* Maximum length of a word */
+#define PI 3.1415926              /* Indiana residents, feel free to */
+                                  /* change it to 3.2 */
 #define DIT '.'
 #define DAH '-'
 #define DEFAULT_WPM   20.0
 #define MIN_WPM        5.0
-#define FARNS_OFF  MIN_WPM - 1.0  /* minimum farnsworth speed */
-#define RISE 2.0
-#define FALL 2.0
+#define FARNS_OFF  MIN_WPM - 1.0  /* farnsworth off */
+#define RISE 10.0
+#define FALL 10.0
 
 #define SAMPLERATE 8000
 //#define SAMPLERATE 44100
 
 #define TRUE 1
 #define FALSE !TRUE
-#define SENT_MSG 7L              /* as defined in common.h */
+#define SENT_MSG 7L               /* as defined in common.h */
 
 /*
  * CWirc stuff
 */
 #if WITH_CWIRC == 1
-#define AUDIOBUF_SIZE 22050   /* Samples */
+#define AUDIOBUF_SIZE 22050   i   /* Samples */
 struct cwirc_extension_api {
    int semid;
    signed short int out_audiobuf[AUDIOBUF_SIZE];
@@ -93,7 +99,7 @@ extern int qid;
 char serdevice[MAX_NAME] = "/dev/ttyS0";       /* default serial device path */
 int  ser_fd = 0;
 char pa_name[MAX_NAME];                        /* shows up on pa vol ctrl */
-int keyMode = 0;                               /* PA, RTS, or CWIRC */
+int keyMode = RTS;                             /* PA, RTS, or CWIRC */
 int doReply = FALSE;                           /* send reply msg when true */
 
 int rts_on = TIOCM_RTS;                        /* for key down */
@@ -101,6 +107,8 @@ int rts_off = 0;                               /* for key up */
 int initialized = 0;                           /* set to 1 by cw_init */
 
 /* for PA */
+static pthread_attr_t cwattr;                 /* thread attributes */
+static pthread_t cwthread = 0;                /* thread for CW output */
 static const pa_sample_spec ss =
    {
       .format = PA_SAMPLE_S16LE,
@@ -110,7 +118,7 @@ static const pa_sample_spec ss =
 
    pa_simple *sp = NULL;
 
-int rtc_rate = 4096;                          /* RTC interupt rate */
+int rtc_rate = 64;                          /* RTC interupt rate */
 
 char *codetable[MAX_LETTERS];
 
@@ -121,20 +129,17 @@ float wpm = DEFAULT_WPM;                      /* current speed to default */
 float farns_wpm = FARNS_OFF;                  /* farnsworth speed to default */
 
 float dit_time = 1200.0 / DEFAULT_WPM;
-float dah_time = 1200.0 / DEFAULT_WPM * 3;     /* dah time in milliseconds */
-float farns_time;                              /* farns time */
+float dah_time = 1200.0 / DEFAULT_WPM * 3;    /* dah time in milliseconds */
+float farns_time;                             /* farns time */
 
-float ic_time = 1200 / DEFAULT_WPM;            /* intra letter time */
-float il_time = 3 * 1200 / DEFAULT_WPM;        /* inter letter time */
-float iw_time = 7 * 1200 / DEFAULT_WPM;        /* inter word time */
+float ic_time = 1200 / DEFAULT_WPM;           /* intra letter time */
+float il_time = 3 * 1200 / DEFAULT_WPM;       /* inter letter time */
+float iw_time = 7 * 1200 / DEFAULT_WPM;       /* inter word time */
 
-float rise = RISE;                      /* risetime in ms */
-float fall = FALL;                      /* fall time in ms */
-float rt = SAMPLERATE / 1000 * RISE;    /* normalized risetime */
-float ft = SAMPLERATE / 1000 * FALL;    /* normalized falltime */
-
-static pthread_t cwthread = 0;                 /* thread for CW output */
-static pthread_attr_t cwattr;                  /* thread attributes */
+float rise = RISE;                            /* risetime in ms */
+float fall = FALL;                            /* fall time in ms */
+float rt = SAMPLERATE / 1000 * RISE;          /* normalized risetime */
+float ft = SAMPLERATE / 1000 * FALL;          /* normalized falltime */
 
 #if WITH_CWIRC == 1
 extern struct cwirc_extension_api *cwirc_api_shm;
@@ -142,8 +147,6 @@ struct cwirc_extension_api *cwirc_api_shm;
 #endif
 
 /* forward declarations internal functions */
-int  cw_set_keyMode (int newmode, char *pgmname);
-void cleanup ();
 void *send_cw (void *p);
 void keySC (char *code);
 void keyRTS (char *code);
@@ -172,7 +175,6 @@ int cw_init (int newmode, char name[], char ser[])
    if (initialized == 1)
    {
       pthread_join (cwthread, NULL); 
-      initialized = 0;
    }
 
    /* check for a name argument */
@@ -189,31 +191,33 @@ int cw_init (int newmode, char name[], char ser[])
       fprintf (stderr, "cw_init Warning: serial device argument is NULL\n");
    }
    else
+   {
       strncpy (serdevice, ser, MAX_NAME - 1) ;    /* copy ser to serdevice */
+   }
 
    /* set newmode to PA or RTS */
    switch (newmode)
    {
       case PA:
          #if DEBUG == 1
-         fprintf (stderr, "cw_init to PA mode\n");
+         fprintf (stderr, "CW_INIT - TO PA MODE\n");
          #endif
          
-         if (cw_set_keyMode (PA, name) != PA)  /* set keyMode to PA */
+         if (cw_set_keyMode (PA, name) != PA)     /* set keyMode to PA */
          {
-            return 0;                          /* set keyMode failed */
+            return 0;                             /* set keyMode failed */
          }
          break;
 
       case RTS:
-         #if DEBUG == 1
-         fprintf (stderr, "cw_init to RTS mode\n");
-         #endif
 
          if ((cw_set_keyMode (RTS, ser)) == RTS)
          {
-            strncpy (serdevice, ser, MAX_NAME - 1); 
+            strncpy (serdevice, ser, MAX_NAME - 1); /* save the ser dev */ 
          }
+         #if DEBUG == 1
+         fprintf (stderr, "CW_INIT - TO RTS MODE ON %s\n", serdevice);
+         #endif
          break;
 
       #if WITH_CWIRC == 1
@@ -231,122 +235,128 @@ int cw_init (int newmode, char name[], char ser[])
    }
 
 
-  /* Initialize the codetable array */
-   for (i = 0; i < MAX_LETTERS; i++)
-      codetable[i] = " ";
-
-   /* Pro Signs and Special */        /* Shift/Alt  */
-   codetable[1]         = "......";   /* F1 = Error */
-   codetable[2]         = ".-.-.";    /* F2 = <AR>  */
-   codetable[3]         = "-.--.";    /* F3 = <KN>  */
-   codetable[4]         = "...-.-";   /* F4 = <SK>  */
-   codetable[5]         = ".-...";    /* F5 = WAIT  */
-   codetable[6]         = "-...-.-";  /* F6 = BREAK */
-
-   /* Numbers */
-   codetable[(int) '1'] = ".----";
-   codetable[(int) '2'] = "..---";
-   codetable[(int) '3'] = "...--";
-   codetable[(int) '4'] = "....-";
-   codetable[(int) '5'] = ".....";
-   codetable[(int) '6'] = "-....";
-   codetable[(int) '7'] = "--...";
-   codetable[(int) '8'] = "---..";
-   codetable[(int) '9'] = "----.";
-   codetable[(int) '0'] = "-----";
-   codetable[216]       = "-----";    /* 216 or 330h is a zero with a slash */
-
-   /* Alphabet */
-   codetable[(int) ' '] = " ";
-   codetable[(int) 'A'] = ".-";
-   codetable[(int) 'B'] = "-...";
-   codetable[(int) 'C'] = "-.-.";
-   codetable[(int) 'D'] = "-..";
-   codetable[(int) 'E'] = ".";
-   codetable[(int) 'F'] = "..-.";
-   codetable[(int) 'G'] = "--.";
-   codetable[(int) 'H'] = "....";
-   codetable[(int) 'I'] = "..";
-   codetable[(int) 'J'] = ".---";
-   codetable[(int) 'K'] = "-.-";
-   codetable[(int) 'L'] = ".-..";
-   codetable[(int) 'M'] = "--";
-   codetable[(int) 'N'] = "-.";
-   codetable[(int) 'O'] = "---";
-   codetable[(int) 'P'] = ".--.";
-   codetable[(int) 'Q'] = "--.-";
-   codetable[(int) 'R'] = ".-.";
-   codetable[(int) 'S'] = "...";
-   codetable[(int) 'T'] = "-";
-   codetable[(int) 'U'] = "..-";
-   codetable[(int) 'V'] = "...-";
-   codetable[(int) 'W'] = ".--";
-   codetable[(int) 'X'] = "-..-";
-   codetable[(int) 'Y'] = "-.--";
-   codetable[(int) 'Z'] = "--..";
-
-   /* Punctuation */
-   codetable[(int) '='] = "-...-";
-   codetable[(int) '/'] = "-..-.";
-   codetable[(int) '?'] = "..--..";
-   codetable[(int) ','] = "--..--";
-   codetable[(int) '.'] = ".-.-.-";
-   codetable[(int) ':'] = "---...";
-   codetable[(int) '\''] = ".----.";
-   codetable[(int) '"'] = ".-..-.";
-   codetable[(int) '_'] = "..--.-";
-   codetable[(int) '('] = "-.--.";
-   codetable[(int) ')'] = "-.--.-";
-   codetable[(int) '#'] = "-.---";
-   codetable[(int) '-'] = "-...-";
-   codetable[(int) '|'] = "...-..";
-   codetable[(int) '\\'] = "-.....";
-   codetable[(int) '*'] = "-----.";
-   codetable[(int) ';'] = "-.-.-.";
-   codetable[(int) '@'] = ".--.-.";
-   codetable[(int) '^'] = "....--.-.";
-   codetable[(int) '$'] = "...-..-";
-   codetable[(int) '!'] = "....-.";
-   codetable[(int) '>'] = "....---.";
-   codetable[(int) ']'] = "....-....";
-   codetable[(int) '['] = "....-..";
-   codetable[(int) '<'] = "....-.-..";
-   codetable[(int) '&'] = "....--.";
-   codetable[(int) '%'] = "....-.--.";
-   codetable[(int) '~'] = "....--";
-   codetable[(int) '+'] = ".-.-.";
-   codetable[(int) '{'] = "....-.--";
-   codetable[(int) '}'] = "....--..-";
-
-   /* Initialize the morse thread */
-   pthread_attr_init (&cwattr);
-   pthread_attr_setdetachstate (&cwattr, PTHREAD_CREATE_JOINABLE);
-
-   /*
-    * trap the following signals for the cleanup function
-    */
-   signal (SIGINT,  (void *)cleanup);
-   signal (SIGTERM, (void *)cleanup);
-   signal (SIGQUIT, (void *)cleanup);
-   signal (SIGSEGV, (void *)cleanup);
-
-   /* create the cwthread and set initialized */
-   rtn = pthread_create (&cwthread, NULL, &send_cw, (void *) "");
-   if (rtn < 0)
+   /* Initialize the codetable array */
+   /* clear the array so unused chars become spaces */
+   if (initialized == 0)
    {
-      fprintf (stdout, "cw_init: pthread_create error\n");
-      return 0;
+      for (i = 1; i < MAX_LETTERS; i++)
+      {
+         codetable[i] = " ";
+      }
+
+      /* Pro Signs and Special */        /* Shift/Alt  */
+      codetable[1]         = "......";   /* F1 = Error */
+      codetable[2]         = ".-.-.";    /* F2 = <AR>  */
+      codetable[3]         = "-.--.";    /* F3 = <KN>  */
+      codetable[4]         = "...-.-";   /* F4 = <SK>  */
+      codetable[5]         = ".-...";    /* F5 = WAIT  */
+      codetable[6]         = "-...-.-";  /* F6 = BREAK */
+
+      /* Numbers */
+      codetable[(int) '1'] = ".----";
+      codetable[(int) '2'] = "..---";
+      codetable[(int) '3'] = "...--";
+      codetable[(int) '4'] = "....-";
+      codetable[(int) '5'] = ".....";
+      codetable[(int) '6'] = "-....";
+      codetable[(int) '7'] = "--...";
+      codetable[(int) '8'] = "---..";
+      codetable[(int) '9'] = "----.";
+      codetable[(int) '0'] = "-----";
+      codetable[216]       = "-----";   /* 216 or 330h is a zero with a slash */
+
+      /* Alphabet */
+      codetable[(int) ' '] = " ";
+      codetable[(int) 'A'] = ".-";
+      codetable[(int) 'B'] = "-...";
+      codetable[(int) 'C'] = "-.-.";
+      codetable[(int) 'D'] = "-..";
+      codetable[(int) 'E'] = ".";
+      codetable[(int) 'F'] = "..-.";
+      codetable[(int) 'G'] = "--.";
+      codetable[(int) 'H'] = "....";
+      codetable[(int) 'I'] = "..";
+      codetable[(int) 'J'] = ".---";
+      codetable[(int) 'K'] = "-.-";
+      codetable[(int) 'L'] = ".-..";
+      codetable[(int) 'M'] = "--";
+      codetable[(int) 'N'] = "-.";
+      codetable[(int) 'O'] = "---";
+      codetable[(int) 'P'] = ".--.";
+      codetable[(int) 'Q'] = "--.-";
+      codetable[(int) 'R'] = ".-.";
+      codetable[(int) 'S'] = "...";
+      codetable[(int) 'T'] = "-";
+      codetable[(int) 'U'] = "..-";
+      codetable[(int) 'V'] = "...-";
+      codetable[(int) 'W'] = ".--";
+      codetable[(int) 'X'] = "-..-";
+      codetable[(int) 'Y'] = "-.--";
+      codetable[(int) 'Z'] = "--..";
+
+      /* Punctuation */
+      codetable[(int) '='] = "-...-";
+      codetable[(int) '/'] = "-..-.";
+      codetable[(int) '?'] = "..--..";
+      codetable[(int) ','] = "--..--";
+      codetable[(int) '.'] = ".-.-.-";
+      codetable[(int) ':'] = "---...";
+      codetable[(int) '\''] = ".----.";
+      codetable[(int) '"'] = ".-..-.";
+      codetable[(int) '_'] = "..--.-";
+      codetable[(int) '('] = "-.--.";
+      codetable[(int) ')'] = "-.--.-";
+      codetable[(int) '#'] = "-.---";
+      codetable[(int) '-'] = "-...-";
+      codetable[(int) '|'] = "...-..";
+      codetable[(int) '\\'] = "-.....";
+      codetable[(int) '*'] = "-----.";
+      codetable[(int) ';'] = "-.-.-.";
+      codetable[(int) '@'] = ".--.-.";
+      codetable[(int) '^'] = "....--.-.";
+      codetable[(int) '$'] = "...-..-";
+      codetable[(int) '!'] = "....-.";
+      codetable[(int) '>'] = "....---.";
+      codetable[(int) ']'] = "....-....";
+      codetable[(int) '['] = "....-..";
+      codetable[(int) '<'] = "....-.-..";
+      codetable[(int) '&'] = "....--.";
+      codetable[(int) '%'] = "....-.--.";
+      codetable[(int) '~'] = "....--";
+      codetable[(int) '+'] = ".-.-.";
+      codetable[(int) '{'] = "....-.--";
+      codetable[(int) '}'] = "....--..-";
+
+      /* Initialize the morse thread */
+      pthread_attr_init (&cwattr);
+      pthread_attr_setdetachstate (&cwattr, PTHREAD_CREATE_JOINABLE);
+
+      /*
+       * trap the following signals when exiting
+       */
+      signal (SIGINT,  (void *)cw_exit);
+      signal (SIGTERM, (void *)cw_exit);
+      signal (SIGQUIT, (void *)cw_exit);
+      signal (SIGSEGV, (void *)cw_exit);
+
+      /* create the cwthread and set initialized */
+      rtn = pthread_create (&cwthread, NULL, &send_cw, (void *) "");
+      if (rtn < 0)
+      {
+         fprintf (stdout, "cw_init: pthread_create error\n");
+         return 0;
+      }
+      initialized = 1;
+      return 1;
    }
-   initialized = 1;
-   return 1;
 }
 
 
 /*
- * cleanup
+ * cw_exit
  * Make sure the sidetone is off and the key is up, then exit
  */
-void cleanup (void)
+void cw_exit (void)
 {
    signal (SIGINT,  SIG_IGN);
    signal (SIGTERM, SIG_IGN);
@@ -358,12 +368,16 @@ void cleanup (void)
    {
       if (ioctl (ser_fd, TIOCMSET, &rts_off) == -1)
       {
-         perror("cleanup: RTS key up failed");
+         perror("cw_exit: RTS key up failed");
       }
    }
 
-   pthread_kill (cwthread, 9);
-   fprintf (stderr, "Thread killed\n");
+   if (pthread_kill (cwthread, 9) == 0)
+   {
+      perror ("try1");
+      fprintf (stderr, "Thread killed\n");
+   }
+
    /* close pulseaudio */
    pa_simple_free (sp);
    exit (73);
@@ -408,7 +422,7 @@ int cw_send_str (char *str)
       return 0;
    }
 
-   /* wait for any current string */
+   /* wait for any current thread */
    pthread_join (cwthread, NULL);
 
    strcpy (s, str);                 /* save the string */
@@ -449,6 +463,7 @@ int cw_send_char (char ch)
 
    str[0] = ch;
    str[1] = '\0';
+
    cw_send_str (str);
    return (1);
 }
@@ -686,26 +701,36 @@ int tonegen (int freq, float len)
 {
    int x, rtn;
    int buf[20000];   //plenty big for a 8wpm dah
-   float val;
+   float val = 0.0;
    int error;
 
-   /* convert len from milliseconds to samples, determine rise/fall time */
-   len = samplerate * len / 1000.0;
+   /* convert len from milliseconds to samples and determine rise/fall time */
+   /* include FUDGE factor */
+   len = samplerate * (len * FUDGE) / 1000.0;
 
    for (x=0; x < len-1; x++)
    {
-      val = 1 + sin(2*PI*freq*x/samplerate);
-      if (x < rt)
+      /* if freq == 0, then quietly fill the buffer with zeros */
+      if (freq == 0)                             /* quiet time */
       {
-         val *= sin(PI*x/(2.0*rt));                 /* rising edge */
+         buf[x] = 0;
+         buf[x] = buf[x] + (buf[x]<<16);
       }
+      else
+      {
+         val = sin(2*PI*freq*x/samplerate);
+         if (x < rt)                             /* within rise time */
+         {
+            val = val * sin(PI*x/(2.0*rt));
+         }
 
-      if (x > (len-ft))
-      {
-         val *= sin(2*PI*(x-(len-ft)+ft)/(4*ft));   /* falling edge */
+         if (x > (len-ft))                       /* within fall time */
+         {
+            val = val * sin(2*PI*(x-(len-ft)+ft)/(4.0*ft));
+         }
+         buf[x] = val * 32000;
+         buf[x] = buf[x] + (buf[x]<<16);         /* second channel */
       }
-      buf[x] = val*8000;
-      buf[x] = buf[x] + (buf[x]<<16);           /* add second channel */
    }
 
    /* now write the buffer */
@@ -796,7 +821,7 @@ int  cw_get_doReply (void)
 
 /*
  *cw_get_serdev - returns the current key mode
-//TJW join str?
+//TJW join thread?
  */
 char *cw_get_serdev (void)
 {
@@ -837,16 +862,10 @@ int cw_set_keyMode (int newmode, char *name)
    }
 #endif
 
-   /* if keyMode is already correct, just return */
-   if (newmode == keyMode)
-   {
-      return keyMode;
-   }
-
    /* if initialized, wait for the current thread */
    if (initialized == 1)
    {
-      //pthread_join (cwthread, NULL);
+      pthread_join (cwthread, NULL);
    }
 
    /* setup the mode */
@@ -860,29 +879,51 @@ int cw_set_keyMode (int newmode, char *name)
                                 name, &ss, NULL, NULL, &error);
             if (sp == NULL)       /* if NULL now, we have an error */
             {
-               perror ("cw_set_keymode: pa_simple_new failed");
+               perror ("cw_set_keyMode: pa_simple_new failed");
                return 0;
             }
          }
+
+         #if DEBUG == 1
+         fprintf (stderr,
+                  "set_keyMode PA wpm=%2.0f/%2.0f dit=%4.1f dah=%4.1f "
+                  "il=%4.1f iw=%4.1f rt/ft=%4.1f\n",
+                  wpm, farns_wpm, dit_time, dah_time, il_time, iw_time, rt);
+         #endif
          break;
 
       case RTS:
+         /* if serdev is NULL, we can't do RTS, so fail */
+         if (strlen (name) == 0)
+         {
+            printf ("Gotta fail\n");
+         }
+
          /* open rtc and find the highest interupt rate */
          rtc_rate = 4096;
          if ((fd = open ("/dev/rtc", O_RDONLY)) < 0)
          {
-            perror ("cw_init: open /dev/rtc failed");
+            perror ("cw_set_keyMode: open /dev/rtc failed");
             exit (1);
          }
          /* find the highest interupt rate */
          /* if 64 is the system default rate, that's kinda slow */
          while (ioctl(fd, RTC_IRQP_SET, rtc_rate) < 0)
          {
-            fprintf (stderr, "cw_init RTC_IRQP_SET %d failed\n", rtc_rate);
+            fprintf (stderr, "cw_set_keyMode RTC_IRQP_SET %d failed\n",
+                     rtc_rate);
             rtc_rate = rtc_rate/2;
          }
          fprintf (stderr, "RTC rate = %d\n", rtc_rate);
          close (fd);   /* close the rtc */
+
+         #if DEBUG == 1
+         fprintf (stderr,
+                  "set_keyMode RTS wpm=%2.0f/%2.0f dit=%4.1f dah=%4.1f "
+                  "il=%4.1f iw=%4.1f rt/ft=%4.1f\n",
+                  wpm, farns_wpm, dit_time, dah_time, il_time, iw_time, rt);
+         #endif
+
          break;
 
       default:
@@ -915,7 +956,6 @@ int cw_set_wpm (int new_wpm)
    float wpm_time;
 
    /* cw_init must be called first */
-//TJW is this true?
    if (initialized == 0)
    {
       fprintf (stderr, "cw_set_wpm: cw_init must be called first.\n");
@@ -926,48 +966,45 @@ int cw_set_wpm (int new_wpm)
     * so wait for the thread
     */
    pthread_join (cwthread, NULL);
+
    /*
     * save the new wpm value if it is >= 5
     */
-   if (new_wpm >= 5)
+   if (new_wpm >= MIN_WPM)    //TJW and =< MAX_WPM
    {
       wpm = new_wpm;
    }
    /*
     * speed is in wpm so we have to calculate the dot-length in
-    * milliseconds using the well-known formula -> dotlength= 1200 / wpm
+    * milliseconds using the well-known formula -> dotlength = 1200 / wpm
     */
-   dit_time = 1200 / wpm;
-   dah_time = 3 * dit_time;
+   dit_time = 1200.0 / wpm;
+   dah_time = 3.0 * dit_time;
+   ic_time = dit_time;
 
-   /* fudge it for a lousy soundcard */
-   if (keyMode == PA)
-   {
-      dit_time = dit_time * FUDGE;
-      dah_time = dah_time * FUDGE;
-   }
-
-   ic_time  = dit_time;
-
-   if ((farns_wpm <= wpm) && (farns_wpm > FARNS_OFF))
+   if ((farns_wpm < wpm) && (farns_wpm > FARNS_OFF))
    { /* do farnsworth timing */
-      farns_time = 1200 / farns_wpm * 50.0; /* time to send a "word" @ fwpm */ 
+      farns_time = 1200 / farns_wpm * 50.0; /* time for a "word" @ fwpm */ 
       wpm_time = 31 * dit_time;             /* PARIS minus iw_time @ wpm */
-      il_time = 3/19.0 * (farns_time - wpm_time);
-      iw_time = 7/19.0 * (farns_time - wpm_time);
+      il_time = 3.0/19.0 * (farns_time - wpm_time);
+      iw_time = 7.0/19.0 * (farns_time - wpm_time);
    }
    else
-   { /* use standard timing */
+   {  /* otherwise use standard timing */
       il_time = 3 * dit_time;
       iw_time = 7 * dit_time;
    }
-   /* rc and ft are the rise and fall times in milliseconds */
+
+   /* rt and ft are the rise and fall times in milliseconds */
    rt = (samplerate / 1000.0) * rise;
    ft = (samplerate / 1000.0) * fall;
 
-   #if DEBUG == 1
-   fprintf (stderr, "libCW dit=%4.1f dah=%4.1f il=%4.1f iw=%4.1f rt/ft=%4.1f\n",
-           dit_time, dah_time, il_time, iw_time, rt);
+#if DEBUG == 1
+   fprintf (stderr,
+            "cw_set_wpm  wpm=%2.0f/%2.0f dit=%4.1f dah=%4.1f "
+            "il=%4.1f iw=%4.1f rt/ft=%4.1f\n",
+            wpm, farns_wpm, dit_time, dah_time,
+            il_time, iw_time, rt);
    #endif
    return wpm;
 }
@@ -1025,6 +1062,14 @@ int cw_set_fwpm (int new_fwpm)
     */
    pthread_join (cwthread, NULL);
 
+   /*
+    * speed is in wpm so we have to calculate the dot-length in
+    * milliseconds using the well-known formula -> dotlength = 1200 / wpm
+    */
+   dit_time = 1200.0 / wpm;
+   dah_time = 3.0 * dit_time;
+   ic_time = dit_time;
+
    if ((new_fwpm < wpm) && (new_fwpm > FARNS_OFF))
    { /* do farnsworth timing */
       farns_wpm = new_fwpm;
@@ -1043,8 +1088,9 @@ int cw_set_fwpm (int new_fwpm)
    ft = (samplerate / 1000.0) * fall;
 
    #if DEBUG == 1
-   fprintf (stderr, "FARNS dit=%4.1f dah=%4.1f il=%4.1f iw=%4.1f\n",
-            dit_time, dit_time*3, il_time, iw_time);
+   fprintf (stderr, "cw_set_fwpm  wpm=%2.0f/%2.0f dit=%4.1f dah=%4.1f "
+            "il=%4.1f iw=%4.1f\n",
+            wpm, farns_wpm, dit_time, dit_time*3, il_time, iw_time);
    fprintf (stderr, "SAMPLERATE = %ld\n", samplerate);
    #endif
    return farns_wpm;
@@ -1070,7 +1116,7 @@ int open_serial ()
    {
       perror("open_serial: can't open the serial device");
    }
-   else
+          else
    {
       /* make sure the key is up */
       if (fd > 0)
@@ -1088,73 +1134,62 @@ int open_serial ()
  */
 int main (int argc, char *argv[])
 {
-   int i;
+   int i, j;
    char d;
    char pgmName[] = "test mode";
    char newdev[] = "/dev/ttyS0";
 
-   /* pick a key mode */
+   /*
+    * select one key mode
+    */
    //int mode = RTS;
    int mode = PA;
    //int mode = CWIRC;
 
    /* initialize libCW */
    cw_init (mode, pgmName, newdev);
-   //cw_init (mode, pgmName, NULL);
 
    /* set some other setup functions */
-   cw_set_doReply (FALSE);
-   cw_set_wpm (20);                      /* set wpm */
-   cw_set_fwpm (20);                     /* set fwpm */
+   cw_set_doReply (FALSE);                 /* turn reply off */
+   cw_set_wpm (20);                        /* set wpm */
+   //cw_set_fwpm (FARNS_OFF);              /* set fwpm */
+   //cw_set_fwpm (10);                     /* set fwpm */
 
    printf ("Enter to start: ");          /* hit rtn when ready */
    d = getchar ();
 
-#if 1
-   /* send paris as 20 strings */
-   for (i=0; i<10; i++)
-   {
-      cw_send_str ("paris ");            /* send "paris" */
+   /*
+    * feel free to add what ever other ways you want to 
+    * test libCW here
+    */
 
-      #if 0
-      if (i % 2 == 0)                    /* altername between PA and RTS */
-      {       /* odd ones */
-         cw_set_fwpm (10);
-      }
-      else
-      {       /* even ones */
-         cw_set_fwpm (FARNS_OFF);
-      }
-      #endif
+#if 0
+   /*
+    * send paris 20 times as chars (twcw style)
+    */
+   for (i=0; i<20; i++)
+   {
+      cw_send_char ('p');
+      cw_send_char ('a');
+      cw_send_char ('r');
+      cw_send_char ('i');
+      cw_send_char ('s');
+      cw_send_char (' ');
    }
 #endif
 
-#if 0
-   /* send paris as 20 words */
-   for (i=0; i<20; i++)
+#if 1
+   /*
+    * send paris 5 times as words (twcw style)
+    */
+   for (i=0; i<5; i++)
    {
       cw_send_word ("paris");
    }
 #endif
 
-#if 0
-   /* send paris as chars - twcw method */
-   for (i=0; i<4; i++)
-   {
-      cw_send_char ('p');
-      cw_send_char ('a');
-      cw_send_char ('r');
-      // cw_set_keyMode(RTS, pgmName);
-      cw_send_char ('i');
-      cw_send_char ('s');
-      cw_send_char (' ');
-      //cw_set_fwpm (i+5);
-   }
-#endif
-
-   /* beep with an 'e' when all done */
-   cw_send_str ("e");
    pthread_join (cwthread, NULL);       /* wait for it before we terminate */
+//cw_exit ();
    return 0;
 }
 #endif   /* END TESTMODE */
