@@ -14,6 +14,7 @@
 
 int  cw_init (int newmode, char *name, char *ser);
 int  cw_get_keyMode (void);
+int  cw_set_keyMode (int newmode, char *pgmname);
 char *cw_get_serdev ();
 int  cw_set_doReply (int mode);
 int  cw_get_doReply (void);
@@ -26,5 +27,6 @@ int  cw_get_freq (void);
 int  cw_send_str  (char *str);
 int  cw_send_word (char *str);
 int  cw_send_char (char ch);
+void cw_exit (void);
 
 /************** end sendCW.h **************/
